/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#ifndef COMMON_H
#define COMMON_H    1

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <math.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <gtk/gtk.h>


/* Definitions of some flags */
#define HOME_LOC_ENTERED    0X0001
#define REMT_LOC_ENTERED    0X0002
#define XPLANET_ENABLED     0X0004
#define HOME_LONG_WEST      0x0008
#define HOME_LATD_SOUTH     0x0010
#define REMT_LONG_WEST      0x0020
#define REMT_LATD_SOUTH     0x0040
#define DEFAULT_HOME        0x0080

/* Return values */
#define ERROR       1
#define SUCCESS     0

/* Size of char arrays (strings) for error messages etc */
#define MESG_SIZE   128

/*-------------------------------------------------------------------*/

typedef struct
{
  /* Home position and name string */
  char
    home_posn[30],
    home_name[25],
    xgridloc_glade[64];

  /* Conversion factor from Km to distance unit */
  double unit_conv;

  /* xplanet command and files */
  char
    xplanet_cmnd[196],
    config_file[64],
    marker_file[64],
    arc_file[64];

} rc_data_t;


/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  define Q_(String) g_strip_context ((String), gettext (String))
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define Q_(String) g_strip_context ((String), (String))
#  define N_(String) (String)
#endif

/*-------------------------------------------------------------------*/

/* Function Prototypes produced by cproto */
/* calculate.c */
void Bearing_Distance(void);
void Read_Home_Position(void);
void Read_Remote_Position(void);
void Read_Home_Locator(void);
void Read_Remote_Locator(void);
void Default_Home_Position(void);
gboolean Locator_Valid(GtkEntry *entry);
gboolean Position_Valid(GtkEntry *entry, gpointer user_data);
/* callbacks.c */
void Error_Dialog(char *message);
void on_main_window_destroy(GObject *object, gpointer user_data);
void on_home_pos_activate(GtkEntry *entry, gpointer user_data);
void on_home_lon_dir_clicked(GtkButton *button, gpointer user_data);
void on_home_lat_dir_clicked(GtkButton *button, gpointer user_data);
void on_home_loc_activate(GtkEntry *entry, gpointer user_data);
void on_remt_pos_activate(GtkEntry *entry, gpointer user_data);
void on_remt_lon_dir_clicked(GtkButton *button, gpointer user_data);
void on_remt_lat_dir_clicked(GtkButton *button, gpointer user_data);
void on_remt_loc_activate(GtkEntry *entry, gpointer user_data);
void on_default_home_clicked(GtkButton *button, gpointer user_data);
void on_enable_xplanet_toggled(GtkToggleButton *togglebutton, gpointer user_data);
void on_quit_clicked(GtkButton *button, gpointer user_data);
void on_pos_changed(GtkEditable *editable, gpointer user_data);
void on_loc_changed(GtkEditable *editable, gpointer user_data);
void on_clear_clicked(GtkButton *button, gpointer user_data);
void on_bad_entry_dialog_destroy(GObject *object, gpointer user_data);
void on_okbutton_clicked(GtkButton *button, gpointer user_data);
gboolean on_main_window_delete(GtkWidget *widget, GdkEvent *event, gpointer user_data);
gboolean on_bad_entry_dialog_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
/* interface.c */
GtkWidget *Builder_Get_Object(GtkBuilder *builder, gchar *name);
GtkWidget *create_main_window(GtkBuilder **builder);
GtkWidget *create_bad_entry_dialog(GtkBuilder **builder);
/* main.c */
int main(int argc, char *argv[]);
/* shared.c */
/* utils.c */
int Load_Line(char *buff, FILE *pfile, char *mesg);
gboolean Load_Config(gpointer data);
int Cleanup(void);
int isFlagSet(int flag);
int isFlagClear(int flag);
void SetFlag(int flag);
void ClearFlag(int flag);
void ToggleFlag(int flag);
void Usage(void);
void Strlcpy(char *dest, const char *src, size_t n);
void Strlcat(char *dest, const char *src, size_t n);
double Atof(const char *nptr);

#endif

