/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#include "main.h"
#include "shared.h"

/* Signal handler */
static void Signal_Handler( int signal );

/* Longitude and Latitude */
extern double
  home_lon, home_lat,
  remt_lon, remt_lat;


int
main( int argc, char *argv[] )
{
  /* Command line option returned by getopt() */
  int option;

  /* New and old actions for sigaction() */
  struct sigaction sa_new, sa_old;

  GtkWidget *main_window = NULL;

  /* Initialize new actions */
  sa_new.sa_handler = Signal_Handler;
  sigemptyset( &sa_new.sa_mask );
  sa_new.sa_flags = 0;

  /* Register function to handle signals */
  sigaction( SIGINT,  &sa_new, &sa_old );
  sigaction( SIGSEGV, &sa_new, 0 );
  sigaction( SIGFPE,  &sa_new, 0 );
  sigaction( SIGTERM, &sa_new, 0 );
  sigaction( SIGABRT, &sa_new, 0 );

  /* Process command line options */
  while( (option = getopt(argc, argv, "hv") ) != -1 )
    switch( option )
    {
      case 'h' : /* Print usage and exit */
        Usage();
        return(0);

      case 'v' : /* Print version */
        puts( PACKAGE_STRING );
        return(0);

      default: /* Print usage and exit */
        Usage();
        exit(-1);

    } /* End of switch( option ) */

  gtk_init(&argc, &argv);

  char rc_fp[512];
  Strlcpy( rc_fp, getenv("HOME"), sizeof(rc_fp) );
  Strlcat( rc_fp, "/.xgridloc/xgridlocrc", sizeof(rc_fp) );
  /* Check for the config file */
  FILE *fp = fopen( rc_fp, "r" );
  if( fp == NULL )
  {
    fprintf( stderr, "xgridloc: cannot access xgridlocrc configuration file.\n" );
    fprintf( stderr, "xgridloc: trying to create xgridloc config directory "
                     "from the installation prefix file tree.\n" );

    /* Find the binary's path (location in file system) */
    char exe_path[256], file_path[288];

    /* Read the file path to xgridloc executable */
    size_t len = sizeof( exe_path );
    int bytes = (int)readlink( "/proc/self/exe", exe_path, len );
    if( bytes <= 0 )
    {
      fprintf( stderr, "xgridloc: cannot read xgridloc binary's location.\n" );
      perror( "/proc/self/exe" );
      exit( -1 );
    }

    /* Remove "/bin/xgridloc" from the path with room for termination */
    bytes -= sizeof( "/bin/xgridloc" ) - 1;
    if( bytes < 1 )
    {
      fprintf( stderr, "xgridloc: cannot create file path to xgridloc/examples.\n" );
      exit( -1 );
    }

    /* Create file path to xgridloc examples directory */
    exe_path[bytes] = '\0';
    Strlcpy( file_path, exe_path, sizeof(file_path) );
    Strlcat( file_path, "/share/xgridloc/examples", sizeof(file_path) );
    fprintf( stderr, "xgridloc: creating xgridloc config directory from: %s\n", file_path );

    /* Create system command to copy xgridloc/examples to ~/.xgridloc */
    char syscmnd[512];
    Strlcpy( syscmnd, "cp -rT ", sizeof(syscmnd) );
    Strlcat( syscmnd, file_path, sizeof(syscmnd) );
    Strlcat( syscmnd, " ", sizeof(syscmnd) );
    Strlcat( syscmnd, getenv("HOME"), sizeof(syscmnd) );
    Strlcat( syscmnd, "/.xgridloc",   sizeof(syscmnd) );
    int ret = system( syscmnd );
    if( ret == -1 )
    {
      fprintf( stderr,"xgridloc: cannot create xgridloc's working directory.\n" );
      perror( file_path );
      exit( -1 );
    }
  } /* if( fp == NULL ) */
  else fclose( fp );

  /* Create file path to xgridloc glade file */
  Strlcpy( rc_data.xgridloc_glade,
      getenv("HOME"), sizeof(rc_data.xgridloc_glade) );
  Strlcat( rc_data.xgridloc_glade,
      "/.xgridloc/xgridloc.glade", sizeof(rc_data.xgridloc_glade) );

  /* Check for the Glade config file */
  fp = fopen( rc_data.xgridloc_glade, "r" );
  if( fp == NULL )
    Strlcpy( rc_data.xgridloc_glade, "/usr/share/xgridloc/xgridloc.glade",
      sizeof(rc_data.xgridloc_glade) );
  else
    fclose(fp);

  /* Create xgridloc main window */
  main_window = create_main_window(&main_window_builder);
  gtk_window_set_title( GTK_WINDOW(main_window), PACKAGE_STRING );

  /* Setup signal callbacks */
  {
    GtkWidget *entry;

    entry = Builder_Get_Object(main_window_builder, "home_lon_deg");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_home_pos_activate), "179");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "179");

    entry = Builder_Get_Object(main_window_builder, "home_lon_min");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_home_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "home_lon_sec");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_home_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "home_lat_deg");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_home_pos_activate), "89");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "89");

    entry = Builder_Get_Object(main_window_builder, "home_lat_min");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_home_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "home_lat_sec");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_home_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "remt_lon_deg");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_remt_pos_activate), "179");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "179");

    entry = Builder_Get_Object(main_window_builder, "remt_lon_min");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_remt_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "remt_lon_sec");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_remt_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "remt_lat_deg");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_remt_pos_activate), "89");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "89");

    entry = Builder_Get_Object(main_window_builder, "remt_lat_min");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_remt_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");

    entry = Builder_Get_Object(main_window_builder, "remt_lat_sec");
    g_signal_connect ((gpointer) entry, "activate",
        G_CALLBACK (on_remt_pos_activate), "59");
    g_signal_connect ((gpointer) entry, "changed",
        G_CALLBACK (on_pos_changed), "59");
  }

  gtk_widget_show(main_window);
  g_idle_add( Load_Config, "" );

  gtk_main();

  return 0;
}

/*------------------------------------------------------------------------*/

/*  Sig_Handler()
 *
 *  Signal Action Handler function
 */

static void Signal_Handler( int signal ) __attribute__ ((noreturn));
static void Signal_Handler( int signal )
{
  Cleanup();

  switch( signal )
  {
    case SIGINT  : fprintf( stderr,  _("\nxgridloc: Exiting via User Interrupt\n") );
           exit(0);
    case SIGSEGV : fprintf( stderr,  _("xgridloc: Segmentation Fault\n") );
           break;
    case SIGFPE  : fprintf( stderr,  _("xgridloc: Floating Point Exception\n") );
           break;
    case SIGABRT : fprintf( stderr,  _("xgridloc: Abort Signal received\n") );
           break;
    case SIGTERM : fprintf( stderr,  _("xgridloc: Termination Request received\n") );
  }

  exit( -1 );

} /* End of Sig_Handler() */

/*------------------------------------------------------------------------*/

